//+------------------------------------------------------------------+
//|                     Smoothed RSI Inverse Fisher Transform EA.mq4 |
//|                                 2013 Hazzabo, Haugesund, Norway |
//|   Indicator formula  2010 Sylvain Vervoort, http://stocata.org/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, Hazzabo"


//--- input parameters
extern double  Lots                = 0.01;
extern int     RsiPeriod           = 9;  // Default 9
extern int     EmaPeriod           = 4;  // Default 4
extern int     InitialTrade        = 0;   // Generate manually at start-up: -1 ->sell, +1 ->Buy, 0 ->No trade
extern int     ActionCandle         = 1;  
extern int     ActionLevelUp        = 12;                                                
extern int     ActionLevelDown      = 88;
extern int     ActionLevelMiddle    = 50;  
extern int     BarCount            = 500;   // Default 500
extern double  TP=0;
extern double  SL=0;
extern double  TrailingStopMode    = 0;            // 0 = Disabled, 1 = Fixed SL, 2 = ATR, 3 = Half Volatility
extern double  TrailingStopSensitivity = 5;
extern double  TrailingStopFactor  = 0.5;          // Applied only if TrailingStopMode = 2 or 3                                              
datetime       OldTime;
datetime       LastAlertTime       = -999999;                                            
int MagicNum;
int open_slippage = 5;
int close_slippage = 10;                     
 
// Buffers used in Fish-calculation
double wma0[], wma1[], wma2[], wma3[], wma4[], wma5[], wma6[], wma7[], wma8[], wma9[];
double ema0[], ema1[], rainbow[], rsi[], srsi[], fish[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
   {
   ArrayResize(wma0, BarCount); ArraySetAsSeries(wma0, true);
   ArrayResize(wma1, BarCount); ArraySetAsSeries(wma1, true);
   ArrayResize(wma2, BarCount); ArraySetAsSeries(wma2, true);
   ArrayResize(wma3, BarCount); ArraySetAsSeries(wma3, true);
   ArrayResize(wma4, BarCount); ArraySetAsSeries(wma4, true);
   ArrayResize(wma5, BarCount); ArraySetAsSeries(wma5, true);
   ArrayResize(wma6, BarCount); ArraySetAsSeries(wma6, true);
   ArrayResize(wma7, BarCount); ArraySetAsSeries(wma7, true);
   ArrayResize(wma8, BarCount); ArraySetAsSeries(wma8, true);
   ArrayResize(wma9, BarCount); ArraySetAsSeries(wma9, true);
   
   MagicNum = 1000 + GetTimeframeConstant(Period()) + GetSymbolConstant(Symbol());
     
   // Reset counter  
   OldTime = Time[0];
   return(0);
   }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
   {
   return(0);
   }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
   {
   int limit = BarCount;  
   
// Resize and shift extra buffers on first and every next bars
   if (Time[0] != OldTime) SyncExtraBuffers();

   
// Prepare for Rainbow-calculation -> partial averages

   for (int i = limit; i >= 0; i--) wma0[i] = iMA(NULL, 0, 2, 0, MODE_LWMA, PRICE_CLOSE, i);
   for (i = limit; i >= 0; i--) wma1[i] = iMAOnArray(wma0, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma2[i] = iMAOnArray(wma1, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma3[i] = iMAOnArray(wma2, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma4[i] = iMAOnArray(wma3, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma5[i] = iMAOnArray(wma4, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma6[i] = iMAOnArray(wma5, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma7[i] = iMAOnArray(wma6, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma8[i] = iMAOnArray(wma7, 0, 2, 0, MODE_LWMA, i);
   for (i = limit; i >= 0; i--) wma9[i] = iMAOnArray(wma8, 0, 2, 0, MODE_LWMA, i);
   
   
// Weigh the averages

   for (i = limit; i >= 0; i--)
      {
      rainbow[i] = (5 * wma0[i] + 4 * wma1[i] + 3 * wma2[i] + 2 * wma3[i] + wma4[i] + wma5[i] + wma6[i] + wma7[i] + wma8[i] + wma9[i]) / 20;
      }   
      
// For debugging: Why is rainbow[ActionCandle] = 0, - when all the wma's [ActionCandle] has a value and ActionCandle = 1????

Comment("\n 2 ",MagicNum, " limit: ", limit," Rainbow: ", rainbow[ActionCandle]," wma0: ", wma0[ActionCandle]," wma1: ", wma1[ActionCandle]," wma2: ", wma2[ActionCandle]," wma3: ", wma3[ActionCandle]); 
      
// Calculate RSI from rainbow 
   for (i = limit; i >= 0; i--) rsi[i] = 0.1 * (iRSIOnArray(rainbow, 0, RsiPeriod, i) - 50);
   
   
// Smooth the RSI with Vervoort zero lag MA
   for (i = limit; i >= 0; i--) ema0[i] = iMAOnArray(rsi, 0, EmaPeriod, 0, MODE_EMA, i);
   for (i = limit; i >= 0; i--) ema1[i] = iMAOnArray(ema0, 0, EmaPeriod, 0, MODE_EMA, i);
   for (i = limit; i >= 0; i--) srsi[i] = ema0[i] + (ema0[i] - ema1[i]);
   
// Calculate Fish
   for (i = limit; i >= 0; i--) fish[i] = ((MathExp(2 * srsi[i]) - 1) / (MathExp(2 * srsi[i]) + 1) + 1) * 50;

// Check to trade
   ProcessBuySellClose();  // HS - Call the Bye, Sell or Close routine
   
   return(0);
   }
   
//+------------------------------------------------------------------+
//| Shift array elements on new bar                                  |
//+------------------------------------------------------------------+
void SyncExtraBuffers()
   {
   for (int i = BarCount - 1; i >= 0; i--)
      {
      wma0[i + 1] = wma0[i];
      wma1[i + 1] = wma1[i];
      wma2[i + 1] = wma2[i];
      wma3[i + 1] = wma3[i];
      wma4[i + 1] = wma4[i];
      wma5[i + 1] = wma5[i];
      wma6[i + 1] = wma6[i];
      wma7[i + 1] = wma7[i];
      wma8[i + 1] = wma8[i];
      wma9[i + 1] = wma9[i];
      }
// Reset counter   
   OldTime = Time[0];
   }
//+------------------------------------------------------------------+
//  Buy or Sell routine by Hazzabo
//+------------------------------------------------------------------+                                                                         
int ProcessBuySellClose()   {                                                                                                                         
//+------------------------------------------------------------------+                                                                          
                                                                                                                                             
    // === Buy processing for crossover UP (indicator line crosses ABOVE signal line) ===                                                     
    bool IsBuy = (fish[ActionCandle] > ActionLevelUp  &&  fish[ActionCandle+1] <= ActionLevelUp)
                  || (fish[ActionCandle]>fish[ActionCandle+1] && fish[ActionCandle+1]<fish[ActionCandle+2])
                  || (InitialTrade == 1);  
                                                                                                                                          
                                                                                                                                                
    // === Sell processing for crossover DOWN (indicator line crosses BELOW signal line) ===                                                  
    bool IsSell = (fish[ActionCandle] < ActionLevelDown  &&  fish[ActionCandle+1] >= ActionLevelDown)
                  || (fish[ActionCandle]<fish[ActionCandle+1] && fish[ActionCandle+1]>fish[ActionCandle+2])
                  || (InitialTrade == -1);  
                                                                                                                                       
    // Closing ?
    bool IsCloseBought = (fish[ActionCandle] < fish[ActionCandle+1]  &&  fish[ActionCandle] <= ActionLevelDown);
    bool IsCloseSold = (fish[ActionCandle] > fish[ActionCandle+1]  &&  fish[ActionCandle] >= ActionLevelDown);

      
   // Control open orders
  int ticket = OrderTicketByMagicNum(MagicNum);
  ControlTrailingStop(ticket);
  
  // Check for exit
  if (ticket > 0)
  {
    if (OrderSelectEx(ticket,SELECT_BY_TICKET,MODE_TRADES)==false) return(0);
    if (OrderType() == OP_BUY && IsCloseBought)
    {
      OrderClose(ticket, OrderLots(), Bid, close_slippage, Red);
    } else if (OrderType() == OP_SELL && IsCloseSold) {
      OrderClose(ticket, OrderLots(), Ask, close_slippage, Red);
    }
  }
  
  
  if (OrderTicketByMagicNum(MagicNum)==0)
  {
    if (IsBuy && !IsSell) {
      EnterBuy();
    } else if(IsSell && !IsBuy) {
      EnterSell();
    } 
  }  
     
                                                                                                                                                                                                                                                                                                                                                                                                             
  return(0);                                                                                                                                    
}  
                                                                                                                                              //

//+------------------------------------------------------------------+
//| Calculate true SL and TP and send order to Buy                                                              |
//+------------------------------------------------------------------+
int EnterBuy()
{
 
  // Calculate true SL/TP
  double TrueSL, TrueTP;
  if (SL > 0) TrueSL = Ask-(SL*Point);
  if (TP > 0) TrueTP = Ask+(TP*Point);
  
  int ret = OrderSendEx(Symbol(), OP_BUY, Lots, Ask, open_slippage*Point, TrueSL, TrueTP, NULL, MagicNum, 0, Yellow);
  InitialTrade=0;
  return(ret);
}


//+------------------------------------------------------------------+
//| Calculate true SL and TP and send order to Sell                                                             |
//+------------------------------------------------------------------+
int EnterSell()
{

  // Calculate true SL/TP
  double TrueSL, TrueTP;
  if (SL > 0) TrueSL = Bid+(SL*Point);
  if (TP > 0) TrueTP = Bid-(TP*Point);
  
  // Send order
  int ret = OrderSendEx(Symbol(), OP_SELL, Lots ,Bid, open_slippage*Point, TrueSL, TrueTP, NULL, MagicNum, 0, Yellow);
  InitialTrade=0;
  return(ret);
  
}


//+------------------------------------------------------------------+
//| Control trailing stop                                            |
//+------------------------------------------------------------------+
void ControlTrailingStop(int ticket)
{
  if (ticket == 0 || TrailingStopMode == 0) return;
  
  double ts;
  if (OrderSelectEx(ticket, SELECT_BY_TICKET, MODE_TRADES)==false) return;
  if (OrderType() == OP_BUY)
  {
    switch (TrailingStopMode)
    {
      case 1: ts = Bid-(Point*SL); break;
      case 2: ts = Low[0] - (TrailingStopFactor * iATR(NULL,0,14,0)); break;
      case 3: ts = Low[0] - (TrailingStopFactor *(High[0]-Low[0])); break;
    }
    if (ts >= OrderStopLoss() + (TrailingStopSensitivity*Point)) 
    {
      OrderModify(ticket, OrderOpenPrice(), ts, OrderTakeProfit(), 0);
    }
    
  }else if(OrderType() == OP_SELL){
  
    switch (TrailingStopMode)
    {
      case 1: ts = Ask+(Point*SL); break;
      case 2: ts = High[0] + (TrailingStopFactor * iATR(NULL,0,14,0)); break;
      case 3: ts = High[0] + (TrailingStopFactor *(High[0]-Low[0])); break;
    }
    if (ts <= OrderStopLoss() - (TrailingStopSensitivity*Point)) 
    {
      OrderModify(ticket, OrderOpenPrice(), ts, OrderTakeProfit(), 0);
    }
  }
  
}


//+------------------------------------------------------------------+
//| Extended OrderSend() for use in multiple pairs                  |
//+------------------------------------------------------------------+
int OrderSendEx(string symbol, int cmd, double volume, double price, int slippage, double stoploss, double takeprofit, string comment, int magic, datetime expiration=0, color arrow_color=CLR_NONE) {
   datetime OldCurTime;
   int timeout=5;
   
   if (!IsTesting()) {
      MathSrand(LocalTime());
      Sleep(MathRand()/6);
   }

   OldCurTime=CurTime();
   while (GlobalVariableCheck("InTrade") && !IsTradeAllowed()) {
      if(OldCurTime+timeout <= CurTime()) {
         Print("Error in OrderSendEx(): Timeout encountered");
         return(0); 
      }
      Sleep(1000);
   }
     
   GlobalVariableSet("InTrade", CurTime());  // set lock indicator
   int ticket = OrderSend(symbol, cmd, volume, price, slippage, stoploss, takeprofit, comment, magic, expiration, arrow_color);
   GlobalVariableDel("InTrade");   // clear lock indicator
   return(ticket);
}


//+------------------------------------------------------------------+
//| Extended OrderSelect()                                           |
//+------------------------------------------------------------------+
bool OrderSelectEx(int index, int select, int pool = MODE_TRADES)
{
  if (OrderSelect(index,select,pool)==true)
  {
    return(true);
  }else{
    Print("Error: Order #", index ," cannot be selected.");
  }
}

//+------------------------------------------------------------------+
//| Get order ticket by magic number                                 |
//+------------------------------------------------------------------+
int OrderTicketByMagicNum(int magic_number) {

  for(int i=0;i<OrdersTotal();i++)
  {
    if (OrderSelectEx(i, SELECT_BY_POS) == false) continue;
    if (OrderMagicNumber() == magic_number) return(OrderTicket());
  }   
      
}


//+------------------------------------------------------------------+
//| Time frame interval appropriation function                       |
//+------------------------------------------------------------------+
int GetTimeframeConstant(int chart_period) {    // To calculate unique MagicNumber
   switch(chart_period) {
      case 1:  // M1
         return(50);
      case 5:  // M5
         return(100);
      case 15:
         return(150);
      case 30:
         return(200);
      case 60:
         return(250);
      case 240:
         return(300);
      case 1440:
         return(350);
      case 10080:
         return(400);
      case 43200:
         return(450);
   }
}


//+------------------------------------------------------------------+
//| Symbol to index                                                  |
//+------------------------------------------------------------------+
int GetSymbolConstant(string symbol) {

	if(symbol=="EURUSD") {	return(1);
	} else if(symbol=="GBPUSD") { return(2);
	} else if(symbol=="USDCHF") {	return(3);
	} else if(symbol=="USDJPY") {	return(4);
	} else if(symbol=="USDCAD") {	return(5);
	} else if(symbol=="AUDUSD") {	return(6);
	} else if(symbol=="CHFJPY") {	return(7);
	} else if(symbol=="EURAUD") {	return(8);
	} else if(symbol=="EURCAD") {	return(9);
	} else if(symbol=="EURCHF") {	return(10);
	} else if(symbol=="EURGBP") {	return(11);
	} else if(symbol=="EURJPY") {	return(12);
   } else if(symbol=="GBPCHF") {	return(13);
	} else if(symbol=="GBPJPY") {	return(14);
	} else if(symbol=="GOLD")   {	return(15);
	} else if(symbol=="#AA")    { return(16);	
   } else if(symbol=="#BA")    {	return(17);
   } else if(symbol=="#GE")    {	return(18);
   } else if(symbol=="#PFE")   {	return(19);
	} else {Print("Warning: Unknowned symbol."); return(20);
	} 

}                                                                                                                                              //
// ============================================================================================================================================ //